/*
 * An example of the plugin for TernAPI-C kernel
 */
#include <stdio.h>
#include "../../tapigate/tapigate.h"

static const char *funcName = "regular3compBIN";


static int numel(void *mat)
{
	return tapi_rbvar_get_m(mat) * tapi_rbvar_get_n(mat);
}

/* Returns function name (to be used by Ruby interpreter) */
extern const char *name()
{
	return funcName;
}

/*
 * Implementation of the function (to be called by Ruby interpreter)
 * Usage:
 *   G = regular3compBIN(x1,x2,x3,:a12=>value,:a23=value,:a13=>value)
 *   x1,x2,x3 -- should have MLSMatrix type
 *   value -- should have Float or Fixnum type
 * Returns:
 *   Gibbs energies, MLSMatrix type
 */
extern void *body(int argc, void **argv, void *context)
{
	int len, i;
	double a12, a13, a23;
	double *x1, *x2, *x3, *G;
	void *a12obj, *a13obj, *a23obj, *Gobj;

	/* Check number of arguments and their types */
	if (argc != 4)
	{
		tapi_rbvar_raise("Invalid number of arguments");
		return tapi_rbvar_get_nil();
	}
	if (tapi_rbvar_get_type(argv[0]) != RBVART_MATRIX ||
	    tapi_rbvar_get_type(argv[1]) != RBVART_MATRIX ||
	    tapi_rbvar_get_type(argv[2]) != RBVART_MATRIX ||
	    tapi_rbvar_get_type(argv[3]) != RBVART_HASH)
	{
		tapi_rbvar_raise("Invalid types of arguments");
		return tapi_rbvar_get_nil();
	}
	/* Check matrices sizes */
	len = numel(argv[0]);
	if (numel(argv[1]) != len || numel(argv[2]) != len)
	{
		tapi_rbvar_raise("xi sizes are not matching");
		return tapi_rbvar_get_nil();
	}
	/* Get model parameters (with checking of their type) */
	a12obj = tapi_rbvar_get_field(argv[3], "a12");
	a13obj = tapi_rbvar_get_field(argv[3], "a13");
	a23obj = tapi_rbvar_get_field(argv[3], "a23");
	if (tapi_rbvar_get_type(a12obj) != RBVART_SCALAR ||
	    tapi_rbvar_get_type(a13obj) != RBVART_SCALAR ||
	    tapi_rbvar_get_type(a23obj) != RBVART_SCALAR)
	{
		tapi_rbvar_raise("aij should be Float or Fixnum");
		return tapi_rbvar_get_nil();
	}	
	a12 = tapi_rbvar_get_scalar(a12obj);
	a13 = tapi_rbvar_get_scalar(a13obj);
	a23 = tapi_rbvar_get_scalar(a23obj);
	/* Create output matrix */
	Gobj = tapi_rbvar_create_matrix(tapi_rbvar_get_m(argv[0]), tapi_rbvar_get_n(argv[1]));
	G = tapi_rbvar_get_matrix(Gobj);
	x1 = tapi_rbvar_get_matrix(argv[0]);
	x2 = tapi_rbvar_get_matrix(argv[1]);
	x3 = tapi_rbvar_get_matrix(argv[2]);
	for (i = 0; i < len; i++)
	{
		G[i] = a12*x1[i]*x2[i]
			+ a13*x1[i]*x3[i]
			+ a23*x2[i]*x3[i];
	}
	return Gobj;
}
